import 'package:ingdao/ingdao.dart';
import 'package:ingdao/dataflow/model/dfwebmodel.dart';
import 'package:ingdao/dataflow/repository/dfwebrepository.dart';
import 'package:ingdao/dataflow/repository/dfsyncrepository.dart';
import 'package:ingdao/dataflow/model/dfdbmodel.dart';
import 'package:ingdao/dataflow/model/dfsyncmodel.dart';
import 'package:ingdao/dataflow/repository/dfdbrepository.dart';
import 'package:nesternship_flut/data/database/registration_db.dart';
import 'package:nesternship_flut/data/web/requests/registration_request.dart';
import 'package:nesternship_flut/data/web/responses/base_response.dart';
import 'package:nesternship_flut/data/web/responses/registration_response.dart';

import 'api_setting.dart';

const REPO_NAME_SQLITEDB = "sqlitedb";
const REPO_NAME_WEB = "web";
const REPO_NAME_ONLINE_SYNC = "onlinesync";

enum RepoType { sqlitedb, web, onlinesync }

extension RepositoryTypeExtension on RepoType {
  String get name {
    switch (this) {
      case RepoType.sqlitedb:
        return REPO_NAME_SQLITEDB;
      case RepoType.web:
        return REPO_NAME_WEB;
      case RepoType.onlinesync:
        return REPO_NAME_ONLINE_SYNC;
      default:
        return null;
    }
  }

  String desp() {
    return this.name;
  }
}

class DataFactory {
  DataFactory._();
  static final instance = DataFactory._();

  Future<DfFactory> getManipulator(RepoType type) async {
    DfFactory dfFactory = DfFactory();

    Map<String, String> headers = Map<String, String>();

    headers["Content-Type"] = "application/json";
    headers["X-DeviceOs"] = "android";
    headers["Version"] = "1";

    //Web
    SslPinningCri
        sslPinningCri; //SslPinningCri(shakeys: appSetting.sslPinningFingerPrints, shatype: SslPinningCriShaType.SHA256);
    DfWebModel modelWeb = DfWebModel(
        name: RepoType.web.name,
        encrypt: ApiSetting.dataEncrypt,
        entities: [
          RegistrationRequest(),
          BaseResponse(),
          RegistrationResponse()
        ],
        serverUrl: ApiSetting.serverUrl,
        commonHeaders: headers,
        sslPinningCri: sslPinningCri);

    DfWebRepository repWeb = DfWebRepository(model: modelWeb);
    dfFactory.registerRepository(RepoType.web.name, repWeb);

    //Database
    DfDbModel modelDb = DfDbModel(
        name: RepoType.sqlitedb.name,
        dbName: ApiSetting.dbName,
        encrypt: ApiSetting.dataEncrypt,
        entities: [RegistrationDb()]);
    DfDbRepository repDb = DfDbRepository(model: modelDb);
    dfFactory.registerRepository(RepoType.sqlitedb.name, repDb);

    //Online Sync
    DfSyncModel modelSync = DfSyncModel(
        name: RepoType.onlinesync.name,
        retryIntervalInSecond: 15 * 60,
        dbRepository: repDb,
        webRepository: repWeb);
    DfSyncRepository repSync = DfSyncRepository(model: modelSync);
    dfFactory.registerRepository(RepoType.onlinesync.name, repSync);

    //Activate
    await dfFactory.activateRepository(type.name);
    return dfFactory;
  }
}
