import 'package:flutter/material.dart';
import 'package:nesternship_flut/data/database/app_database.dart';
import 'package:nesternship_flut/ui/screen/landing_screen.dart';
import 'package:nesternship_flut/ui/screen/splash_screen.dart';
import 'package:nesternship_flut/ui/style/app_theme.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/ui/component/nui_app.dart';
import 'package:nui_core/ui/nui_display_util.dart';
import 'package:nui_core/ui/nui_sharedpref_util.dart';
import 'package:nui_database/nui_database.dart';
import 'package:nui_error_handler/nui_error_handler.dart';
import 'package:nui_log/nui_log.dart';
import 'package:nui_media/nui_media.dart';
import 'package:nui_web/nui_web.dart';
import 'ui/screen/landing_screen.dart';

final errorHandler = () => NUIErrorHandlerBuilder()
        .module("app")
        .appVersion(appConfig().versionName)
        .showErrorOnEncounter(false)
        .redirect((context, actionCode, errorMapping) {
      switch (actionCode) {
        case "404":
          {
            //Todo something here
            break;
          }
        case "401":
          {
            //Todo something here
            break;
          }
        case "1001":
          {
            //Todo something here
            break;
          }
      }
    });

final logging = () => NUILogBuilder()
    .appVersion(appConfig().versionName)
    .appCenterAndroidKey(configField<String>("appCenterKeyAndroid"))
    .appId(appConfig().appId);

const String APP_MODULE = "Nesternship";

void main() {
  runNUIAppWithLog(
      NUIApp(
        buildType: BuildType.DEV,
        title: "Nesternshipt",
        themeData: (context) => theme(context),
        overlayStyle: (context) => NUIOverlayStyle(darkStatusBarIcon: false),
        preAppTask: (context) async {
          //Build shared preference
          final pref = NUISharedPrefBuilder().setInitials({
            "appName": "Nesternship",
          }).build();

          //Todo: If location usage is required, uncomment this
          //Build locator
//      final geoLocator = NUIGeoLocatorBuilder()
//          .checkLocationSetting(false)
//          .setLocationUpdateCallback((position, areaCode) {
//      }).build();


          final media = NUIMediaBuilder().build();

          //Build http web instance
          final web = NUIWebBuilder()
              .headers(() => {
                    "Authorization":
                        "Bearer eyJhbGciOiJIUzUxMiJ9.eyJzdWIiOiI2NiIsInVzZXIiOnsicm9sZSI6IlJPTEVfVVNFUiIsImlkIjoiNjYifSwiaWF0IjoxNTk1MjIxMjczLCJleHAiOjE1OTUzMDc2NzN9.edhC0tJm5E_1huvFuF9FJQPBoEiAzjvQBXIFegkdKkxbOJc0UjA1Aa2rbshgqFP3RbY9OuaY3qB2kLebeX3cFA",
                    "X-DeviceOs": "web"
                  })
              .module(APP_MODULE)
              .baseURL(appConfig().baseURL)
              .errorHandler(errorHandler)
              .statusMessages({
            404: "Oops, this page you're looking for is not found",
            500: "Oops, seems like something went wrong"
          }).build();

          //Build database
          final db = NUIDatabaseBuilder()
              .readyCallback((success, message) {
                logNUI("Main", "Open database with success : $success");
              })
              .model(AppDatabase())
              .build();

          await db;
//      await geoLocator;
          await pref;

          logNUI("Main", "Finish loading builders");
          return true;
        },
//    homeScreen: (context) => !GeneralBloc.isLoggedIn() ? LoginScreen() : Scaffold(body: LandingScreen()),
        homeScreen: (context) => Scaffold(body: LandingScreen()),
        loadingScreen: (context) => SplashScreen(),
      ),
      logBuilder: logging,
      errorHandlerBuilder: errorHandler);
}
