import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:line_awesome_flutter/line_awesome_flutter.dart';
import 'package:nesternship_flut/bloc/calendar_bloc.dart';
import 'package:nesternship_flut/bloc/general_bloc.dart';
import 'package:nesternship_flut/bloc/training_course_bloc.dart';
import 'package:nesternship_flut/data/model/banner.dart';
import 'package:nesternship_flut/ui/list/app_list_repo.dart';
import 'package:nesternship_flut/ui/list/skeletal/listitem_training_skeletal.dart';
import 'package:nesternship_flut/ui/screen/pdf_viewer_screen.dart';
import 'package:nesternship_flut/ui/screen/video_viewer_screen.dart';
import 'package:nesternship_flut/ui/screen/webview_screen.dart';
import 'package:nesternship_flut/ui/screen_codes.dart';
import 'package:nesternship_flut/ui/style/app_theme.dart';
import 'package:nesternship_flut/ui/custom/pageview_indicator.dart';
import 'package:nui_animation/nui_animation.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_core/ui/component/nui_persistent_bottom_sheet_widget.dart';
import 'package:nui_core/ui/component/nui_state_widget.dart';
import 'package:visibility_detector/visibility_detector.dart';

import 'notification/notification_listing_screen.dart';

class HomeScreen extends NUIScreen {
  HomeScreen()
      : super(
            overlayStyle: NUIOverlayStyle(darkStatusBarIcon: true),
            content: (context) => Container(
                width: double.infinity,
                height: double.infinity,
                child: MainBody()),
            screenCode: ScreenCodes.landing);
}

class MainBody extends StatelessWidget {
  final PageController _pageController =
      PageController(viewportFraction: 1.0, initialPage: 0);
  final TrainingCourseBloc _bloc = TrainingCourseBloc();
  final GeneralBloc _generalBloc = GeneralBloc();
  final _currentPageNotifier = ValueNotifier<int>(0);
  final NUIBlocData<bool> _statusBarNotifier = NUIBlocData();
  final NUIBlocData<bool> _titleCoverNotifier = NUIBlocData();
  final NUIBlocData<double> _bannerOffsetNotifier = NUIBlocData();
  final NUIBlocData<double> _pageIndicatorOffsetNotifier = NUIBlocData();

  @override
  Widget build(BuildContext context) {
    return Container(
      width: double.infinity,
      height: double.infinity,
      child: Stack(
        children: [
          Container(
            width: double.infinity,
            height: double.infinity,
            color: getColor(context, white),
          ),
          Container(
              width: double.infinity,
              height: screenHeight(context) * 0.5,
              color: getColor(context, grayOutline),
              child: Stack(
                children: [
                  NUIBlocStream<double>(
                    bloc: _bannerOffsetNotifier,
                    stream: (context, status, ratio) => Container(
                      width: screenWidth(context),
                      height: status == BlocStatus.COMPLETED
                          ? (screenHeight(context) * 0.5) * ratio
                          : (screenHeight(context) * 0.5),
                      child: PageView.builder(
                          controller: _pageController,
                          onPageChanged: (index) {
                            //Update indicator
                            _currentPageNotifier.value = index;
                          },
                          itemCount: _bloc.getBannerSliderImages().length,
                          itemBuilder: (context, position) {
                            return BannerSliderImage(
                                image: _bloc.getBannerSliderImages()[position]);
                          }),
                    ),
                  ),
                  NUIBlocStream<double>(
                    bloc: _pageIndicatorOffsetNotifier,
                    stream: (context, status, ratio) => Positioned(
                        right: 16,
                        bottom: status == BlocStatus.COMPLETED
                            ? (31 + screenHeight(context) * 0.05) +
                                ((screenHeight(context) * 0.5) * (1 - ratio))
                            : (31 + screenHeight(context) * 0.05),
                        child: PageViewIndicator(
                          currentPageNotifier: _currentPageNotifier,
                          dotSpacing: 16,
                          itemCount: _bloc.getBannerSliderImages().length,
                          size: 8,
                          selectedSize: 12,
                          borderColor: getColor(context, white),
                          borderWidth: 1,
                          dotColor: NUIColors.NUITransparent,
                          selectedDotColor: getColor(context, white),
                          selectedBorderColor: getColor(context, white),
                        )),
                  )
                ],
              )),
          Container(
            margin:
                EdgeInsets.only(top: NUIDisplayUtil.statusBarHeight(context)),
            child: NUIDraggableSheet(
              collapseRatio: 0.55,
              expandAcceptMargin: 0.01,
              expandRatio: 1,
              scrollListener: (noti, expanded, collapsed) {
                logNUI("LandingScreen",
                    "Expanded : $expanded, collapsed : $collapsed, extent : ${noti.extent}");
                _updateTopCover(expanded);
                _updateBannerOffset(noti);
              },
              decoration: NUIBoxDecoration(
                  color: getColor(context, white),
                  borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(20),
                      topRight: Radius.circular(20))),
              child: ExploreBottomSheetContent(),
              header: NUIPersistentHeader(
                height: 69,
                child: NUIBlocStream<bool>(
                  bloc: _titleCoverNotifier,
                  stream: (context, status, expanded) => Container(
                    width: double.infinity,
                    height: 69,
                    decoration: NUIBoxDecoration(
                        color: getColor(context, white),
                        borderRadius: BorderRadius.only(
                            topLeft: Radius.circular(20),
                            topRight: Radius.circular(20)),
                        shadowRadius: expanded == true ? 10 : 0,
                        shadowColor: getColor(context, toolbarShadow)),
                    child: NUIColumn(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        mainAxisSize: MainAxisSize.min,
                        children: [
                          AnimatedOpacity(
                              opacity: expanded == true ? 1.0 : 0.0,
                              duration: Duration(milliseconds: 250),
                              child: Container(
                                height: 15,
                                width: double.infinity,
                                color: getColor(context, white),
                              )),
                          Container(
                            height: 44,
                            alignment: Alignment.centerLeft,
                            padding: EdgeInsets.only(left: 20, right: 20),
                            child: AnimatedSwitcher(
                                duration: Duration(milliseconds: 200),
                                child: expanded != true
                                    ? SizedBox(
                                        key: Key("bigTitle"),
                                        width: 200,
                                        child: Text(
                                            getString("homeScreenTitle"),
                                            style: getFont(context,
                                                size: 32, color: textBlue)))
                                    : SizedBox(
                                        key: Key("smallTitle"),
                                        width: 200,
                                        child: Text(
                                            getString("homeScreenTitle"),
                                            style: getFont(context,
                                                size: 24, color: textBlue)))),
                          ),
                          Divider(height: 10, color: NUIColors.NUITransparent)
                        ]),
                  ),
                ),
              ),
            ),
          ),
          NUIColumn(
              crossAxisAlignment: CrossAxisAlignment.end,
              children: [
                NUIBlocStream<bool>(
                    bloc: _statusBarNotifier,
                    stream: (context, status, data) => AnimatedOpacity(
                          duration: Duration(milliseconds: 250),
                          opacity: data == true ? 1.0 : 0.0,
                          child: Container(
                            width: double.infinity,
                            height: NUIDisplayUtil.statusBarHeight(context) + 5,
                            color: getColor(context, white),
                          ),
                        )),
                Padding(
                  padding: EdgeInsets.only(top: 10),
                  child: NotificationBell(
                      count: _generalBloc.getNotificationCount()),
                )
              ],
              divider: Divider(height: 5, color: NUIColors.NUITransparent))
        ],
      ),
    );
  }

  void _updateTopCover(bool expanded) {
    _statusBarNotifier.notify(expanded);
    _titleCoverNotifier.notify(expanded);
  }

  void _updateBannerOffset(DraggableScrollableNotification noti) {
    final maxRatio = 0.7;
    final ratio = ((noti.maxExtent - noti.extent).toDouble() /
        (noti.maxExtent - noti.minExtent));
    final finalRatio = ratio >= maxRatio ? ratio : maxRatio;

    final diff = 1 - finalRatio;
    final slowedDiff = diff * 0.6;
    final slowedRatio = 1 - slowedDiff;
    _bannerOffsetNotifier.notify(slowedRatio);
    _pageIndicatorOffsetNotifier.notify(slowedRatio);
  }
}

class BannerSliderImage extends StatefulWidget {
  BannerSliderImage({this.image});
  final BannerImage image;
  @override
  _BannerSliderImageState createState() => _BannerSliderImageState();
}

class _BannerSliderImageState extends State<BannerSliderImage> {
  String id;
  bool showingText = false;

  @override
  void initState() {
    super.initState();
    id = randomUUID();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        width: double.infinity,
        height: double.infinity,
        child: Stack(
          children: [
            Container(
              width: double.infinity,
              height: double.infinity,
              decoration: BoxDecoration(
                  image: DecorationImage(
                      image: loadImageProvider(widget.image.bannerImage),
                      fit: BoxFit.cover)),
            ),
            Positioned(
              bottom: 31 + screenHeight(context) * 0.05 + 12 + 16,
              right: 0,
              child: Padding(
                padding: EdgeInsets.only(right: 16, left: 40),
                child: SizedBox(
                  width: screenWidth(context) * 0.7,
                  child: VisibilityDetector(
                    key: Key(id),
                    onVisibilityChanged: (info) {
                      if (info.visibleFraction > 0.7) {
                        if (showingText != true) {
                          setState(() {
                            showingText = true;
                          });
                        }
                      } else {
                        if (showingText != false) {
                          setState(() {
                            showingText = false;
                          });
                        }
                      }
                    },
                    child: AnimatedOpacity(
                      duration: Duration(milliseconds: 250),
                      opacity: showingText == true ? 1.0 : 0.0,
                      child: NUIColumn(
                        crossAxisAlignment: CrossAxisAlignment.end,
                        divider: Divider(
                            height: 16, color: NUIColors.NUITransparent),
                        children: [
                          Text(widget.image.title,
                              style: getFont(context,
                                  size: 32, color: textWhite, isBold: true),
                              textAlign: TextAlign.end,
                              maxLines: 2,
                              overflow: TextOverflow.ellipsis),
                          Text(widget.image.description,
                              style:
                                  getFont(context, size: 16, color: textWhite),
                              textAlign: TextAlign.end,
                              maxLines: 4,
                              overflow: TextOverflow.ellipsis),
                        ],
                      ),
                    ),
                  ),
                ),
              ),
            )
          ],
        ));
  }
}

class ExploreBottomSheetContent extends StatelessWidget {
  final TrainingCourseBloc _bloc = TrainingCourseBloc();

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: NUIBoxDecoration(
        color: getColor(context, white),
//        borderRadius: BorderRadius.only(topLeft: Radius.circular(20), topRight: Radius.circular(20))
      ),
      child: NUIColumn(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisSize: MainAxisSize.max,
        children: [
          Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Padding(
                padding: EdgeInsets.only(left: 20, right: 20),
                child: Text(getString("homeScreenDesc"),
                    style: getFont(context, size: 16, color: textDarkGray)),
              ),
              Divider(
                height: 15,
                color: NUIColors.NUITransparent,
              ),
              NUIRow(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Padding(
                    padding: EdgeInsets.only(left: 20, right: 20),
                    child: Text("Mandatory Trainings",
                        style:
                            getFont(context, size: 14, color: textLightGray)),
                  ),
                  NUIBouncingAnimation(
                    scaleFactor: 0.5,
                    child: Padding(
                      padding: EdgeInsets.only(left: 20, right: 20),
                      child: NUIRow(
                        divider: VerticalDivider(
                          width: 5,
                          color: NUIColors.NUITransparent,
                        ),
                        children: [
                          Text("More",
                              style: getFont(context,
                                  size: 12, color: accentBright, isBold: true)),
                          Icon(
                            LineAwesomeIcons.arrow_right,
                            color: getColor(context, accentBright),
                            size: 17,
                          )
                        ],
                      ),
                    ),
                  ),
                ],
              ),
              SizedBox(
                height: 310,
                width: screenWidth(context),
                child: NUIAnimatedList(
                  repo: AppListRepo(),
                  enablePagination: false,
                  enableSwipeRefresh: false,
                  orientation: Axis.horizontal,
                  skeletalCount: 1,
                  itemAnimation: NUIListItemAnimation.FADE,
                  paginationInflater: NUILoadingListItem(
                      shadowRadius: 5,
                      inflater: (context) => LITrainingCourseSkeletal()),
                  skeletalInflater: NUILoadingListItem(
                      shadowRadius: 5,
                      inflater: (context) => LITrainingCourseSkeletal()),
                  middleDivider: 5,
                  padding: const EdgeInsets.only(
                      bottom: 10, top: 10, left: 15, right: 15),
                  initialList: [],
                  bloc: _bloc.coursesBloc,
                  clickListener: (data, position, clickType) {},
                ),
              ),
              Divider(
                height: 15,
                color: NUIColors.NUITransparent,
              ),
              NUIRow(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Padding(
                    padding: EdgeInsets.only(left: 20, right: 20),
                    child: Text("Live Video Trainings",
                        style:
                            getFont(context, size: 14, color: textLightGray)),
                  ),
                  NUIBouncingAnimation(
                    scaleFactor: 0.5,
                    child: Padding(
                      padding: EdgeInsets.only(left: 20, right: 20),
                      child: NUIRow(
                        divider: VerticalDivider(
                          width: 5,
                          color: NUIColors.NUITransparent,
                        ),
                        children: [
                          Text("More",
                              style: getFont(context,
                                  size: 12, color: accentBright, isBold: true)),
                          Icon(
                            LineAwesomeIcons.arrow_right,
                            color: getColor(context, accentBright),
                            size: 17,
                          )
                        ],
                      ),
                    ),
                  ),
                ],
              ),
              SizedBox(
                height: 310,
                width: screenWidth(context),
                child: NUIAnimatedList(
                  repo: AppListRepo(),
                  enablePagination: false,
                  enableSwipeRefresh: false,
                  skeletalCount: 1,
                  middleDivider: 5,
                  orientation: Axis.horizontal,
                  itemAnimation: NUIListItemAnimation.FADE,
                  paginationInflater: NUILoadingListItem(
                      shadowRadius: 5,
                      inflater: (context) => LITrainingCourseSkeletal()),
                  skeletalInflater: NUILoadingListItem(
                      shadowRadius: 5,
                      inflater: (context) => LITrainingCourseSkeletal()),
                  padding: const EdgeInsets.only(
                      bottom: 10, top: 10, left: 15, right: 15),
                  initialList: [],
                  bloc: _bloc.coursesBloc2,
                  clickListener: (data, position, clickType) {},
                ),
              ),
              Divider(
                height: 15,
                color: NUIColors.NUITransparent,
              ),
              NUIRow(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Padding(
                    padding: EdgeInsets.only(left: 20, right: 20),
                    child: Text("Live Video Trainings",
                        style:
                            getFont(context, size: 14, color: textLightGray)),
                  ),
                  NUIBouncingAnimation(
                    scaleFactor: 0.5,
                    child: Padding(
                      padding: EdgeInsets.only(left: 20, right: 20),
                      child: NUIRow(
                        divider: VerticalDivider(
                          width: 5,
                          color: NUIColors.NUITransparent,
                        ),
                        children: [
                          Text("More",
                              style: getFont(context,
                                  size: 12, color: accentBright, isBold: true)),
                          Icon(
                            LineAwesomeIcons.arrow_right,
                            color: getColor(context, accentBright),
                            size: 17,
                          )
                        ],
                      ),
                    ),
                  ),
                ],
              ),
              SizedBox(
                height: 310,
                width: screenWidth(context),
                child: NUIAnimatedList(
                  repo: AppListRepo(),
                  enablePagination: false,
                  enableSwipeRefresh: false,
                  skeletalCount: 1,
                  middleDivider: 5,
                  orientation: Axis.horizontal,
                  itemAnimation: NUIListItemAnimation.FADE,
                  paginationInflater: NUILoadingListItem(
                      shadowRadius: 5,
                      inflater: (context) => LITrainingCourseSkeletal()),
                  skeletalInflater: NUILoadingListItem(
                      shadowRadius: 5,
                      inflater: (context) => LITrainingCourseSkeletal()),
                  padding: const EdgeInsets.only(
                      bottom: 10, top: 10, left: 15, right: 15),
                  initialList: [],
                  bloc: _bloc.coursesBloc3,
                  clickListener: (data, position, clickType) {},
                ),
              )
            ],
          ),
        ],
      ),
    );
  }
}

class NotificationBell extends StatelessWidget {
  NotificationBell({this.count: 0});
  final GeneralBloc bloc = GeneralBloc();
  final CalendarBloc calBloc = CalendarBloc();

  final int count;
  @override
  Widget build(BuildContext context) {
    return Container(
      margin: EdgeInsets.only(right: 15, left: 15),
      child: Stack(
        children: [
          Visibility(
            visible: count != null && count > 0,
            child: Positioned(
                right: 0,
                child: Container(
                  padding:
                      EdgeInsets.only(left: 3, right: 3, top: 2, bottom: 2),
                  decoration: NUIBoxDecoration(
                      color: getColor(context, accentBright),
                      borderRadius: BorderRadius.circular(4)),
                  child: Text(bloc.formatNotificationCount(count),
                      style: getFont(context,
                          size: 8, color: textWhite, isBold: true)),
                )),
          ),
          GestureDetector(
            onTap: () {
              showNotifications(context);
            },
            child: Padding(
              padding: EdgeInsets.only(left: 5, right: 5, top: 2, bottom: 2),
              child: Icon(
                LineAwesomeIcons.bell,
                color: getColor(context, black),
                size: 30,
              ),
            ),
          )
        ],
      ),
    );
  }
}
