import 'dart:developer';
import 'dart:math';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:carousel_slider/carousel_controller.dart';
import 'package:carousel_slider/carousel_options.dart';
import 'package:carousel_slider/carousel_slider.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:nesternship_flut/bloc/landing_page_bloc.dart';
import 'package:nesternship_flut/bloc/registration_bloc.dart';
import 'package:nesternship_flut/ui/custom/apple_login_button.dart';
import 'package:nesternship_flut/ui/custom/google_login_button.dart';
import 'package:nesternship_flut/ui/custom/yellow_background_button.dart';
import 'package:nesternship_flut/ui/screen/login_screen.dart';
import 'package:nesternship_flut/ui/screen/registration_steps_screen.dart';

import 'package:nui_core/nui_core.dart';
import 'package:visibility_detector/visibility_detector.dart';

import '../../bloc/training_course_bloc.dart';
import '../../data/model/banner.dart';
import '../custom/pageview_indicator.dart';
import '../screen_codes.dart';
import '../style/app_colors.dart';
import '../style/app_theme.dart';
import 'registration_main_screen.dart';

class LandingScreen extends StatelessWidget {
  final CarouselController buttonCarouselController = CarouselController();
  final LandingPageBloc _bloc = new LandingPageBloc();
  final RegistrationBloc registrationBloc = new RegistrationBloc();

  @override
  Widget build(BuildContext context) {
    registrationBloc.registerAccount();

    return NUIScreen(
        overlayStyle: NUIOverlayStyle(darkStatusBarIcon: false),
        content: (context) => Scaffold(
              body: Container(
                  // decoration: BoxDecoration(
                  //   image: DecorationImage(
                  //     image: AssetImage('assets/images/landing_bg.png'),
                  //     fit: BoxFit.cover
                  //   )
                  // ),
                  width: double.infinity,
                  height: double.infinity,
                  child: NUIColumn(
                    mainAxisSize: MainAxisSize.max,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: [
                      LandingPageSlider(),
                    ],
                  )),
            ),
        screenCode: ScreenCodes.splash);
  }
}

class LandingPageSlider extends StatefulWidget {
  @override
  _LandingPageSliderState createState() => _LandingPageSliderState();
}

class _LandingPageSliderState extends State<LandingPageSlider> {
  int _currentIndex = 0;
  final LandingPageBloc _bloc = LandingPageBloc();
  List<BannerImage> imageList;

  void callback() {
    setState(() {});
  }

  @override
  void initState() {
    super.initState();
    imageList = _bloc.getLandingPageSliderImages();
  }

  @override
  Widget build(BuildContext context) {
    final double height = MediaQuery.of(context).size.height;
    final double width = MediaQuery.of(context).size.width;
    return Expanded(
      child: Stack(children: [
        Container(
            child: Stack(
          children: imageList
              .map((item) => Center(child: SliderImage(item, width, height)))
              .toList(),
        )),
        CarouselSlider(
          options: CarouselOptions(
              height: height,
              viewportFraction: 1.0,
              enlargeCenterPage: false,
              autoPlay: false,
              autoPlayInterval: Duration(seconds: 3),
              autoPlayAnimationDuration: Duration(milliseconds: 800),
              autoPlayCurve: Curves.fastOutSlowIn,
              onPageChanged: (index, reason) {
                setState(() {
                  _currentIndex = index;
                });
              }),
          items: imageList
              .map((item) => Center(
                      child: Stack(
                    children: [
                      //image
                      //slider description
                      SliderDetails(item, this.callback)
                    ],
                  )))
              .toList(),
        ),
        SliderIndicatorAndBottomButtons(selectedIndex: _currentIndex)
      ]),
    );
  }
}

class SliderImage extends StatefulWidget {
  final BannerImage currentImage;
  final double sliderImageWidth;
  final double sliderImageHeight;

  SliderImage(this.currentImage, this.sliderImageWidth, this.sliderImageHeight);

  @override
  _SliderImageState createState() => _SliderImageState();
}

class _SliderImageState extends State<SliderImage> {
  @override
  Widget build(BuildContext context) {
    return Opacity(
      opacity: widget.currentImage.transparency,
      child: Image.asset('assets/images/landing_bg.png',
          fit: BoxFit.cover,
          height: widget.sliderImageHeight,
          width: widget.sliderImageWidth),
    );
  }
}

class SliderDetails extends StatefulWidget {
  final BannerImage image;
  final Function callback;

  SliderDetails(this.image, this.callback);

  @override
  _SliderDetailsState createState() => _SliderDetailsState();
}

class _SliderDetailsState extends State<SliderDetails> {
  double _sliderDetailsTransparency = 1.0;

  @override
  Widget build(BuildContext context) {
    VisibilityDetectorController.instance.updateInterval = Duration.zero;

    return VisibilityDetector(
      key: Key(widget.image.title),
      onVisibilityChanged: (VisibilityInfo visibilityInfo) {
        if (visibilityInfo.key.toString().contains(widget.image.title)) {
          if (mounted) {
            setState(() {
              _sliderDetailsTransparency = visibilityInfo.visibleFraction;
              widget.image.transparency = visibilityInfo.visibleFraction;
            });
            widget.callback();
          }
        }
      },
      child: Column(
        mainAxisAlignment: MainAxisAlignment.end,
        children: [
          Divider(
            height: 28,
            color: Colors.transparent,
          ),
          Opacity(
            opacity: _sliderDetailsTransparency,
            child: Text(widget.image.title,
                style: getFont(context, size: 24, color: white, isBold: false)),
          ),
          Divider(
            height: 20,
            color: Colors.transparent,
          ),
          Opacity(
            opacity: _sliderDetailsTransparency,
            child: Container(
              margin: EdgeInsets.fromLTRB(16, 0, 16, 0),
              child: Text(
                widget.image.description,
                style: getFont(context, size: 16, color: white, isBold: false),
                maxLines: 4,
                overflow: TextOverflow.ellipsis,
                textAlign: TextAlign.center,
              ),
            ),
          ),
          Divider(
            height: 280,
            color: Colors.transparent,
          )
        ],
      ),
    );
  }
}

class SliderIndicatorAndBottomButtons extends StatefulWidget {
  final int selectedIndex;
  final LandingPageBloc _bloc = LandingPageBloc();

  SliderIndicatorAndBottomButtons({@required this.selectedIndex});

  @override
  _SliderIndicatorAndBottomButtonsState createState() =>
      _SliderIndicatorAndBottomButtonsState();
}

class _SliderIndicatorAndBottomButtonsState
    extends State<SliderIndicatorAndBottomButtons> {
  ValueNotifier<int> _currentPageNotifier;

  @override
  void initState() {
    super.initState();
    _currentPageNotifier = ValueNotifier<int>(widget.selectedIndex);
  }

  @override
  Widget build(BuildContext context) {
    _currentPageNotifier.value = widget.selectedIndex;

    return NUIColumn(
      mainAxisSize: MainAxisSize.max,
      mainAxisAlignment: MainAxisAlignment.end,
      children: [
        Expanded(
            child: Container(
          margin: EdgeInsets.fromLTRB(0, 30, 0, 0),
          alignment: Alignment.center,
          child: Image.asset(
            'assets/images/logo_white.png',
          ),
        )),
        Row(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.end,
          children: [
            PageViewIndicator(
              currentPageNotifier: _currentPageNotifier,
              dotSpacing: 16,
              itemCount: widget._bloc.getLandingPageSliderImages().length,
              size: 8,
              selectedSize: 12,
              borderColor: getColor(context, white),
              borderWidth: 1,
              dotColor: NUIColors.NUITransparent,
              selectedDotColor: getColor(context, white),
              selectedBorderColor: getColor(context, white),
            )
          ],
        ),
        Divider(
          height: 186,
          color: Colors.transparent,
        ),
        GoogleLoginButton(text: "Sign up with Google", onTap: () {}),
        Divider(
          height: 16,
          color: Colors.transparent,
        ),
        AppleLoginButton(text: "Sign up with Apple ID", onTap: () {}),
        Divider(
          height: 16,
          color: Colors.transparent,
        ),
        YellowBackgroundButton(
          text: getString("landingScreenButtonDesc"),
          onTap: () {
            push(context, RegistrationMainScreen(),
                transition: PageTransition.FADE);
          },
        ),
        Divider(
          height: 10,
          color: Colors.transparent,
        ),
        FlatButton(
          onPressed: () {
            push(context, LoginScreen(),
                transition: PageTransition.FADE,
                duration: Duration(seconds: 1));
          },
          child: Text(
            getString('landingScreenToLoginButton'),
            style: getFont(context, size: 16, color: white, isBold: true),
          ),
          padding: EdgeInsets.all(10),
        ),
        Divider(
          height: 16,
          color: Colors.transparent,
        ),
      ],
    );
  }
}
