import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_slidable/flutter_slidable.dart';
import 'package:nesternship_flut/bloc/notification_bloc.dart';
import 'package:nesternship_flut/data/model/country_model.dart';
import 'package:nesternship_flut/data/model/notification.dart';
import 'package:nesternship_flut/data/model/search_criteria.dart';
import 'package:nesternship_flut/ui/custom/notification_listing_toolbar.dart';
import 'package:nesternship_flut/ui/custom/search_toolbar.dart';
import 'package:nesternship_flut/ui/list/skeletal/listitem_notification_skeletal.dart';
import 'package:nesternship_flut/ui/list/skeletal/listitem_reg_country_skeletal.dart';
import 'package:nesternship_flut/ui/screen_codes.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_core/ui/component/nui_search_text.dart';
import 'package:nesternship_flut/ui/list/app_list_repo.dart';
import '../../style/app_theme.dart';
import 'notification_detail_screen.dart';

void showNotifications(BuildContext context) {
  push(context, NotificationListingScreen());
}

class NotificationListingScreen extends StatefulWidget {
  @override
  _NotificationListingScreenState createState() =>
      _NotificationListingScreenState();
}

class _NotificationListingScreenState extends State<NotificationListingScreen> {
  final NotificationBloc bloc = NotificationBloc();
  NUIListController listController;

  @override
  void initState() {
    super.initState();
    listController = NUIListController(onLoadComplete: () {});
  }

  @override
  Widget build(BuildContext context) {
    return NUIScreen(
      overlayStyle: NUIOverlayStyle(darkStatusBarIcon: true),
      screenCode: ScreenCodes.notificationList,
      content: (context) => Scaffold(
        body: Container(
          width: double.infinity,
          height: double.infinity,
          color: getColor(context, white),
          alignment: Alignment.center,
          child: NUIColumn(
            mainAxisSize: MainAxisSize.max,
            children: [
              NotificationListingToolbar(),
              Expanded(
                  child: NUIAnimatedHeaderList(
                repo: AppListRepo(),
                enablePagination: true,
                enableSwipeRefresh: true,
                controller: listController,
                skeletalCount: 2,
                itemAnimation: NUIListItemAnimation.FADE,
                skeletalInflater: NUILoadingListItem(
                    inflater: (context) => LINotificationSkeletal()),
                padding: const EdgeInsets.only(bottom: 8, top: 8),
                initialList: [],
                bloc: bloc.userNotifications(),
                clickListener: (data, position, clickType) {
                  if (data is UserNotification) {
                    showNotificationDetail(context, data);
                  }
                },
              ))
            ],
          ),
        ),
      ),
    );
  }
}
