import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nesternship_flut/bloc/registration_bloc.dart';
import 'package:nesternship_flut/data/model/country_model.dart';
import 'package:nesternship_flut/ui/form/country_selection_form.dart';
import 'package:nesternship_flut/ui/screen/registration_steps_screen.dart';
import 'package:nesternship_flut/ui/style/app_theme.dart';
import 'package:nui_animation/nui_animation.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_form/nuif_form.dart';

import '../country_selection_screen.dart';

class RegistrationStep1Screen extends StatefulWidget {
  RegistrationStep1Screen(this.registrationController);
  final RegistrationController registrationController;
  @override
  _RegistrationStep1ScreenState createState() =>
      _RegistrationStep1ScreenState();
}

class _RegistrationStep1ScreenState extends State<RegistrationStep1Screen> {
  NUIForm form;
  Country selectedCountry = Country();
  RegistrationBloc regBloc = RegistrationBloc();
  String countryValue = "Select a Country";

  @override
  void initState() {
    super.initState();
    form = countrySelectionForm(context, () {
      // push(context)
    });
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      color: Colors.transparent,
      child: Container(
        width: double.infinity,
        height: double.infinity,
        padding: EdgeInsets.only(left: 32, right: 32),
        child: NUIColumn(
          mainAxisSize: MainAxisSize.max,
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisAlignment: MainAxisAlignment.center,
          divider: Divider(height: 40, color: NUIColors.NUITransparent),
          children: [
            Text("In which country are you conducting your Nesternship?",
                style: getFont(context, size: 21, color: textWhite)),
            GestureDetector(
              child: _renderForm(context, selectedCountry),
              onTap: () async {
                final selectedCountry = await pushForResult<Country>(
                    context, CountrySelectionScreen(),
                    transition: PageTransition.BOTTOM_TOP);
                if (selectedCountry.name != null) {
                  this.selectedCountry = selectedCountry;
                  widget.registrationController.setValue(selectedCountry);

                  form.fillInValues({
                    "edtCountry": NUIFormValue(
                        inpId: "edtCountry",
                        value: NUIFormSelection(
                            code: this.selectedCountry.name,
                            value: this.selectedCountry.name))
                  });

                  setState(() {});
                }
              },
            ),
            WorldImage()
          ],
        ),
      ),
    );
  }

  Widget _renderForm(BuildContext context, Country theCountry) {
    final comp = form.updateItem<NUIFTextItem>("edtCountry", (item) {
      item.controller?.text = selectedCountry?.name;
      return item;
    });

    return NUIFormContent(form: form);
  }
}

class WorldImage extends StatefulWidget {
  @override
  _WorldImageState createState() => _WorldImageState();
}

class _WorldImageState extends State<WorldImage> with TickerProviderStateMixin {
  AnimationController controller;
  Animation <double> animation;

  @override
  void initState() {
    super.initState();
    controller = AnimationController(
        duration: const Duration(milliseconds: 3000),
        vsync: this,
        value: 0,
        lowerBound: 0,
        upperBound: 1
    );
    animation = CurvedAnimation(parent: controller, curve: Curves.fastOutSlowIn);

    controller.forward();
  }

  @override
  dispose() {
    controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return FadeTransition(
      opacity: animation,
      child: AspectRatio(
        aspectRatio: (351 / 142.83),
        child: loadSvg("map.svg", color: null),
      ),
    );
  }
}

