import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:nesternship_flut/ui/custom/screen_header.dart';
import 'package:nesternship_flut/ui/custom/yellow_background_button.dart';
import 'package:nesternship_flut/ui/form/registration_form.dart';
import 'package:nesternship_flut/ui/screen/registration_steps_screen.dart';
import 'package:nesternship_flut/ui/screen_codes.dart';
import 'package:nesternship_flut/ui/style/app_theme.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_form/nuif_form.dart';

class RegistrationMainScreen extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return NUIScreen(
        overlayStyle: NUIOverlayStyle(darkStatusBarIcon: false),
        content: (context) => Scaffold(
          body: Container(
            width: double.infinity,
            height: double.infinity,
            decoration: defaultGradientBackground(context),
            child: SingleChildScrollView(
              child: SizedBox(
                height: MediaQuery.of(context).size.height,
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    ScreenHeader(
                      true, 
                      context,
                      backIconColor: getColor(context, accentBright)
                    ),
                    Divider(
                        height: 40,
                        color: NUIColors.NUITransparent),
                    Align(
                      alignment: Alignment.center,
                      child: Image.asset(
                        'assets/images/logo.png',
                        color: Colors.white,
                        height: 54,
                      ),
                    ),
                    Divider(
                        height: 67,
                        color: NUIColors.NUITransparent),
                    Padding(
                      padding: EdgeInsets.only(left: 32, right: 32),
                      child: Text(
                          getString("registrationScreenTitle"),
                          style: getFont(context,
                              size: 21, color: textWhite)),
                    ),
                    Divider(
                        height: 40,
                        color: NUIColors.NUITransparent),
                    RegistrationMainFormBody(),
                  ]
                )
              )
            )
          ),
        ),
        screenCode: ScreenCodes.splash);
  }
}

class RegistrationMainFormBody extends StatefulWidget {
  @override
  _RegistrationMainFormBodyState createState() => _RegistrationMainFormBodyState();
}

class _RegistrationMainFormBodyState extends State<RegistrationMainFormBody> {
  NUIForm form;
  int curValue = 3;
  int _currentValue = 15;
  bool validated;

  @override
  void initState() {
    super.initState();
    form = RegistrationForm().emailPasswordForm(context, (id, value, form) {
      logNUI("IFormPreview", "Text changed on $id, cur value : $value");
    }, (id, clickType, form) {
      logNUI("IFormPreview", "Button clicked on $id, click type : $clickType");
      validated = form.validate(false);
      final values = form.extractValues();
    }, (id, selection, selected, form) {
      logNUI("IFormPreview",
          "Item selected on $id, cur value : ${selection.value}, selected : $selected");
    });
  }

  Widget _renderForm(BuildContext context) {
    return NUIFormContent(form: form);
  }

  void _getFormValues() {
    final validated = form.validate(false);
    if (!validated) return;
    final values = form.extractValues();
  }

  @override
  Widget build(BuildContext context) {
    return Expanded(
      child: NUIColumn(
        mainAxisSize: MainAxisSize.max,
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Container(
            width: double.infinity,
            padding: EdgeInsets.only(left: 32, right: 32),
            child: _renderForm(context),
          ),

          Container(
            margin: EdgeInsets.fromLTRB(0, 0, 0, 60),
            child: YellowBackgroundButton(
                text: getString('registrationScreenContinueButton'),
                onTap: () {
                  bool valid = form.validate(false);
                  if (valid) {
                    push(context, RegistrationStepsScreen());
                  }
                }),
          )
        ],
      ),
    );
  }
}
