import 'dart:math';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:line_awesome_flutter/line_awesome_flutter.dart';
import 'package:nesternship_flut/ui/custom/yellow_background_button.dart';
import 'package:nesternship_flut/ui/screen/registration/registration_step1.dart';
import 'package:nesternship_flut/ui/screen/registration/registration_step5.dart';
import 'package:nesternship_flut/ui/screen/registration/registration_step3.dart';
import 'package:nesternship_flut/ui/screen/registration/registration_step2.dart';
import 'package:nesternship_flut/ui/screen/registration_success_screen.dart';
import 'package:nesternship_flut/ui/screen_codes.dart';
import 'package:nesternship_flut/ui/screen/registration/registration_step4.dart';
import 'package:nesternship_flut/ui/style/app_theme.dart';
import 'package:nesternship_flut/ui/custom/stepper_line_indicator.dart';
import 'package:nui_animation/nui_animation.dart';
import 'package:nui_core/nui_core.dart';

//Validation controller on each screen before the user can continue to next step
class RegistrationController {
  RegistrationController({this.optional: false});
  dynamic _value;
  bool optional;
  Function _valueUpdateListener;

  void setValue(dynamic value) {
    _value = value;
    print(_value);
    _valueUpdateListener();
  }

  dynamic getValue() {
    return _value;
  }

  bool hasEntered() {
    return _value != null;
  }
}

class RegistrationStepsScreen extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return NUIScreen(
      overlayStyle: NUIOverlayStyle(darkStatusBarIcon: false),
      content: (context) => ContentBody(),
      screenCode: ScreenCodes.registrationSteps);
  }
}

class ContentBodyController {
  Function onBackPressCallback;
  void onBackPress() {
    if (onBackPressCallback != null) onBackPressCallback();
  }
}

class ContentBody extends StatefulWidget {
  @override
  _ContentBodyState createState() => _ContentBodyState();
}

class _ContentBodyState extends State<ContentBody> {
  int curPage = 0;
  PageController pageController = PageController();
  final _currentPageNotifier = ValueNotifier<int>(0);
  final NUIBlocData<int> buttonNotifier = NUIBlocData();
  final NUIBlocData<int> _pageTextNotifier = NUIBlocData();
  final NUIBlocData<int> _skipPageNotifier = NUIBlocData();
  ContentBodyController controller = ContentBodyController();

  RegistrationController controllerStep1 = RegistrationController();
  RegistrationController controllerStep2 = RegistrationController();
  RegistrationController controllerStep3 =
      RegistrationController(optional: true);
  RegistrationController controllerStep4 =
      RegistrationController(optional: true);
  RegistrationController controllerStep5 =
      RegistrationController(optional: true);

  List<RegistrationController> getControllers() {
    return [
      controllerStep1,
      controllerStep2,
      controllerStep3,
      controllerStep4,
      controllerStep5
    ];
  }

  void _updateStepValidation() {
    buttonNotifier.notify(curPage);
  }

  @override
  void initState() {
    super.initState();
    for (RegistrationController cont in getControllers()) {
      cont._valueUpdateListener = () {
        _updateStepValidation();
      };
    }
    controller?.onBackPressCallback = () {
      previousPage();
    };
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Container(
        decoration: NUIBoxDecoration(
            gradient: NUILinearGradient(colors: [
          getColor(context, accentBlueDark),
          getColor(context, accentBlueBright)
        ], orientation: GradientOrientation.T_B)),
        child: SingleChildScrollView(
          child: SizedBox(
            height: screenHeight(context),
            child: NUIColumn(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                NUIStatusBar(),
                Divider(height: 15, color: NUIColors.NUITransparent),
                StepperHeader(
                  currentPageNotifier: _currentPageNotifier,
                  pageTextNotifier: _pageTextNotifier,
                  bodyController: controller,
                  skipPageNotifier: _skipPageNotifier,
                ),
                Divider(height: 15, color: NUIColors.NUITransparent),
                Expanded(
                  child: PageView(
                    physics: NeverScrollableScrollPhysics(),
                    controller: pageController,
                    children: [
                      NUITabViewContent(
                          child: RegistrationStep1Screen(controllerStep1)),
                      NUITabViewContent(
                          child: RegistrationStep2Screen(controllerStep2)),
                      NUITabViewContent(
                          child: RegistrationStep3Screen(controllerStep3)),
                      NUITabViewContent(
                          child: RegistrationStep4Screen(controllerStep4)),
                      NUITabViewContent(
                          child: RegistrationStep5Screen(controllerStep5)),
                    ],
                  )
                ),
                Divider(height: 15, color: NUIColors.NUITransparent),
                NUIBlocStream<int>(
                  bloc: buttonNotifier,
                  stream: (context, state, data) => NUIColumn(
                    mainAxisSize: MainAxisSize.min,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: [
                      Opacity(
                        opacity: validatePage(curPage) == true ? 1.0 : 0,
                        child: YellowBackgroundButton(
                            text: getString('registrationScreenContinueButton'),
                            onTap: () {
                              if (validatePage(curPage)) {
                                nextPage();
                              }
                            }),
                      ),
                      curPage > 1
                          ?
                      FlatButton(
                        onPressed: () {
                          skipPage();
                        },
                        child: Text(
                          getString('registrationStepScreenSkipButton'),
                          style: getFont(context, size: 16, color: white, isBold: true),
                        ),
                        padding: EdgeInsets.fromLTRB(30, 20, 30, 20),
                      )
                          :
                      SizedBox(
                        height: 59,
                      )
                    ],
                  ),
                ),
                Divider(height: 32, color: NUIColors.NUITransparent)
              ],
            ),
          ),
        ),
      ),
    );
  }

  bool optionalPage(int position) {
    return getControllers()[position].optional == true;
  }

  bool validatePage(int position) {
    return getControllers()[position].hasEntered();
  }

  void skipPage() {
    nextPage(validate: false);
  }

  void nextPage({bool validate: true}) {
    final validated = validatePage(curPage);

    if (validate && !validated) return;

    if (curPage != 4) {
      //Not the last page yet
      curPage += 1;
      buttonNotifier.notify(curPage);
      pageController.jumpToPage(curPage);
      _pageTextNotifier.notify(curPage + 1);
      _skipPageNotifier.notify(curPage + 1);
      _currentPageNotifier.value = curPage;
    } else {
      //Last page
      popAndPush(context, RegistrationSuccessScreen(),
          transition: PageTransition.FADE,
          duration: Duration(milliseconds: 250));
    }
  }

  void previousPage() {
    if (curPage != 0) {
      //Not the last page yet
      curPage -= 1;
      buttonNotifier.notify(curPage);
      pageController.jumpToPage(curPage);
      _pageTextNotifier.notify(curPage + 1);
      _skipPageNotifier.notify(curPage + 1);
      _currentPageNotifier.value = curPage;
    } else {
      pop(context);
    }
  }
}

class BottomButtonNavigation extends StatefulWidget {
  @override
  _BottomButtonNavigationState createState() => _BottomButtonNavigationState();
}

class _BottomButtonNavigationState extends State<BottomButtonNavigation> {
  @override
  Widget build(BuildContext context) {
    return Container();
  }
}

class StepperHeader extends StatefulWidget {
  StepperHeader(
      {this.currentPageNotifier, this.pageTextNotifier, this.bodyController, this.skipPageNotifier});
  final ValueNotifier<int> currentPageNotifier;
  final NUIBlocData<int> pageTextNotifier;
  final NUIBlocData<int> skipPageNotifier;
  final ContentBodyController bodyController;

  @override
  _StepperHeaderState createState() => _StepperHeaderState();
}

class _StepperHeaderState extends State<StepperHeader> {
  int currentStep = 1;
  int totalStep = 5;

  @override
  Widget build(BuildContext context) {
    return Container(
      width: double.infinity,
      padding: EdgeInsets.only(left: 10, right: 10, bottom: 15),
      child: NUIColumn(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisSize: MainAxisSize.min,
        divider: Divider(
          height: 8,
          color: NUIColors.NUITransparent,
        ),
        children: [
          NUIRow(
            mainAxisSize: MainAxisSize.max,
            crossAxisAlignment: CrossAxisAlignment.center,
            divider:
                VerticalDivider(width: 10, color: NUIColors.NUITransparent),
            children: [
              NUIBouncingAnimation(
                  child: Padding(
                    padding: EdgeInsets.fromLTRB(6, 0, 10, 0),
                    child: Icon(LineAwesomeIcons.arrow_left,
                        color: getColor(context, accentBright), size: 28),
                  ),
                  onTap: () {
                    widget.bodyController?.onBackPress();
                  }),
              NUIBlocStream<int>(
                bloc: widget.pageTextNotifier,
                stream: (context, state, data) => RichText(
                  textAlign: TextAlign.start,
                  text: TextSpan(
                      style: getFont(context, size: 14.0, color: textWhite),
                      children: [
                        TextSpan(text: "Step "),
                        TextSpan(
                            text: (data ?? 1).toString(),
                            style: getFont(context,
                                isBold: true, color: textWhite, size: 14)),
                        TextSpan(text: " out of "),
                        TextSpan(
                            text: totalStep.toString(),
                            style: getFont(context,
                                isBold: true, color: textWhite, size: 14)),
                      ]),
                ),
              ),
              Expanded(
                  child: SizedBox(
                height: 2,
              )),
              NUIBlocStream<int>(
                bloc: widget.skipPageNotifier,
                stream: (context, state, data) =>
                //if is page 3 and beyond
                max(data ?? 1, 3) == data
                  ?

                FlatButton(
                  onPressed: () {
                    popAllAndPush(context, RegistrationSuccessScreen());
                  },
                  child: Text(
                    getString('registrationScreenSkipAllButton'),
                    style: getFont(context, size: 14, color: accentBright, isBold: true),
                  ),
                  padding: EdgeInsets.all(10),
                )
                  :
                //material codebase uses this to represent empty widget
                SizedBox.shrink(),
              )
            ],
          ),
          Container(
            width: double.infinity,
            padding: EdgeInsets.only(left: 58, right: 10),
            child: StepperLineIndicator(
              currentPageNotifier: widget.currentPageNotifier,
              size: 4,
              selectedSize: 8,
              itemCount: 5,
              color: getColor(context, white).withOpacity(0.6),
              selectedColor: getColor(context, white),
            ),
          )
        ],
      ),
    );
  }
}
