import 'package:advance_pdf_viewer/advance_pdf_viewer.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:line_awesome_flutter/line_awesome_flutter.dart';
import 'package:nesternship_flut/ui/custom/app_video_player.dart';
import 'package:nesternship_flut/ui/screen_codes.dart';
import 'package:nui_animation/nui_button_animation.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_core/ui/component/nui_pdf_viewer.dart';
import '../style/app_theme.dart';

void showVideoViewer(BuildContext context, {@required String title, @required String videoUrl}){
  push(context, VideoViewerScreen(videoUrl, title), transition: PageTransition.LEFT_RIGHT);
}

class VideoViewerScreen extends StatefulWidget {
  VideoViewerScreen(this.videoUrl, this.title);
  final String videoUrl;
  final String title;
  @override
  _VideoViewerScreenState createState() => _VideoViewerScreenState ();
}


class _VideoViewerScreenState extends State<VideoViewerScreen> {

  @override
  Widget build(BuildContext context){
    return NUIScreen(
      overlayStyle: NUIOverlayStyle(darkStatusBarIcon: true),
      screenCode: ScreenCodes.videoViewer,
      content: (context) => Scaffold(
          body:Container(
              color: getColor(context, white),
              alignment: Alignment.center,
              child:NUIColumn(
                  mainAxisSize: MainAxisSize.max,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisAlignment: MainAxisAlignment.center,
                  children:[
                    _Toolbar(widget.title),
                    Expanded(
                        child:Container(
                          height: double.infinity,
                          width: double.infinity,
                          child: Stack(
                            children: [
                              VideoContentBody(widget.videoUrl),
                              Container(
                                height: 10,
                                width: double.infinity,
                                decoration: NUIBoxDecoration(
                                    gradient: NUILinearGradient(
                                        colors: [
                                          getColor(context, toolbarShadow),
                                          NUIColors.NUITransparent
                                        ],
                                        orientation: GradientOrientation.T_B
                                    )
                                ),
                              )
                            ],
                          ),
                        )

                    )
                  ]
              )
          )
      ),
    );
  }
}


class VideoContentBody extends StatelessWidget {
  VideoContentBody(this.videoUrl);
  final String videoUrl;
  @override
  Widget build(BuildContext context) {
    return Container(
      width: double.infinity,
      height: double.infinity,
      child: NUIColumn(
        mainAxisSize: MainAxisSize.max,
        children: [
          AppVideoPlayer(videoUrl)
        ],
      ),
    );
  }
}



class _Toolbar extends StatefulWidget {
  _Toolbar(this.title);
  final String title;
  @override
  _ToolbarState createState() => _ToolbarState();
}

class _ToolbarState extends State<_Toolbar> {

  @override
  Widget build(BuildContext context) {
    return Container(
      width: double.infinity,
      decoration: toolbarWhiteDecoration(context),
      child: Column(
        children: [
          NUIStatusBar(),
          Container(
            height: 48,
            width: double.infinity,
            child: Padding(
              padding: EdgeInsets.only(left: 8, right: 8),
              child: NUIRow(
                mainAxisSize: MainAxisSize.max,
                crossAxisAlignment: CrossAxisAlignment.center,
                divider:
                VerticalDivider(width: 10, color: NUIColors.NUITransparent),
                children: [
                  NUIBouncingAnimation(
                      child: Padding(
                        padding: EdgeInsets.all(10),
                        child: Icon(
                          LineAwesomeIcons.angle_left,
                          color: getColor(context, accentBright),
                          size: 28,
                        ),
                      ),
                      onTap: () {
                        pop(context);
                      }),
                  Expanded(
                    child: Text(
                        widget.title,
                        style:getFont(context, size:16, color:textDarkGray)
                    ),
                  ),
//                  NUIBouncingAnimation(
//                      child: Padding(
//                          padding: EdgeInsets.all(10),
//                          child: Text(getString("toolbarNotificationClearAll"), style: getFont(context, size: 12, color: accentBright, isBold: true))
//                      ),
//                      onTap: () {
//                        pop(context);
//                      }),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }
}